==> pimucHA-0.8.3.zip: pimucHA-0.8.3/piHAcore/servers/tcprxevt.py <==
# ----------------------------------------------------------------------------
#       Copyright (C) 2013-2014 Huynh Vi Lam  <domovilam@gmail.com>
#
#       This file is part of pimucha.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import logging,select,socket,sys,threading
from piHAlibs import (addr2host,coroutine)

logger = logging.getLogger()

if sys.hexversion > 0x030000F0:
    import queue as Queue
else:
    import Queue
EVTNETq = Queue.Queue()
QEmpty = Queue.Empty

@coroutine
def evtnetqueue():
    while True:
        evt = (yield)
        if isinstance(evt,list):
            data = '|'.join(evt)
        else:
            data = evt
        EVTNETq.put_nowait(data)

class TCPRXServer(object):
    def __init__(self,addr=None):
        self.host,self.port = addr2host(defaddr=('0.0.0.0','11000'),addr=addr)
        self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server.setblocking(0)
        self.type = 'SRV'
        self.name = 'TCPRXServer'
        self.evtnetq = EVTNETq
        self.servercmds = {'STARTS':self.starts,'STOPS':self.stops}
        self.inputs = [self.server]
        self.outputs = []
        self.cnxqueues = {}
        self.setup = False
        self.stop = False
 
    def tsetup(self):
        try:
            self.server.bind((self.host,self.port))
            self.server.listen(5)
            self.setup = True
        except:
            pass

    def closecnx(self,s):
        if s in self.outputs:
            self.outputs.remove(s)
        if s in self.inputs:
            self.inputs.remove(s)
        s.close()
        del self.cnxqueues[s]

    def starts(self):
        tt = threading.Thread(target=self.serve)
        tt.name = 'TCPRXServer_' + self.name
        tt.daemon = True
        tt.start()
        print ('TCPRXServer started and active...')

    def serve(self):
        print ('Listening at address %s on port number %s...\n' % (self.host, self.port))
        LF = b"\n"
        while not self.stop:
            try:
                evt = self.evtnetq.get_nowait()
            except QEmpty:
                evt = None
            if evt:
                logger.debug("Getting evt from queue %s ", repr(evt))
                for q in self.cnxqueues:
                    if isinstance(evt,list):
                        for e in evt:
                            self.cnxqueues[q].put_nowait(e + LF)
                    else:
                        self.cnxqueues[q].put_nowait(evt + LF)

            readable, writable, exceptional = select.select(self.inputs, self.outputs, self.inputs)
            for s in readable:
                if s is self.server:
                    cnx, caddr = s.accept()
                    logger.debug("New connection from %s", caddr)
                    cnx.setblocking(0)
                    self.inputs.append(cnx)
                    self.outputs.append(cnx)
                    self.cnxqueues.setdefault(cnx)
                    self.cnxqueues[cnx] = Queue.Queue()
                else:
                    try:
                        data = s.recv(1024)
                        caddr = s.getpeername()
                    except Exception as e:	#disconnected
                        logger.error("%s : %s", repr(e.__class__), str(e))
                        data = 'QUIT'
                    if data != "" :
                        logger.debug('Receiving "%s" from %s',data,caddr)
                    if data.upper() == 'QUIT':
                        self.closecnx(s)
            for s in writable:
                if s not in self.outputs: break
                try:
                    next_msg = self.cnxqueues[s].get_nowait()
                    s.send(next_msg)                   
                except QEmpty:
                    pass
            for s in exceptional:
                self.closecnx(s)

    def stops(self):
        for s in self.outputs:
            self.closecnx(s)
        self.server.close()
        self.stop = True
        print ('TCPRXServer shutting down...')

==> pimucHA-0.8.4.zip: pimucHA-0.8.4/piHAcore/servers/tcprxevt.py <==
# ----------------------------------------------------------------------------
#       Copyright (C) 2013-2014 Huynh Vi Lam  <domovilam@gmail.com>
#
#       This file is part of pimucha.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import logging,select,socket,sys,threading
from piHAlibs import (addr2host,coroutine)

logger = logging.getLogger()

if sys.hexversion > 0x030000F0:
    import queue as Queue
else:
    import Queue
EVTNETq = Queue.Queue()
QEmpty = Queue.Empty

@coroutine
def evtnetqueue():
    while True:
        evt = (yield)
        if isinstance(evt,list):
            data = '|'.join(evt)
        else:
            data = evt
        EVTNETq.put(data)

class TCPRXServer(object):
    def __init__(self,addr=None):
        self.host,self.port = addr2host(defaddr=('0.0.0.0','11000'),addr=addr)
        self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server.setblocking(0)
        self.type = 'SRV'
        self.name = 'TCPRXServer'
        self.evtnetq = EVTNETq
        self.servercmds = {'STARTS':self.starts,'STOPS':self.stops}
        self.inputs = [self.server]
        self.outputs = []
        self.cnxqueues = {}
        self.setup = False
        self.stop = False
 
    def tsetup(self):
        try:
            self.server.bind((self.host,self.port))
            self.server.listen(5)
            self.setup = True
        except:
            pass

    def closecnx(self,s):
        if s in self.outputs:
            self.outputs.remove(s)
        if s in self.inputs:
            self.inputs.remove(s)
        s.close()
        del self.cnxqueues[s]

    def starts(self):
        tt = threading.Thread(target=self.serve)
        tt.name = 'TCPRXServer_' + self.name
        tt.daemon = True
        tt.start()
        print ('TCPRXServer started and active...')

    def serve(self):
        print ('Listening at address %s on port number %s...\n' % (self.host, self.port))
        LF = b"\n"
        while not self.stop:
            try:
                evt = self.evtnetq.get()
            except QEmpty:
                evt = None
            if evt:
                logger.debug("Getting evt from queue %s ", repr(evt))
                for q in self.cnxqueues:
                    if isinstance(evt,list):
                        for e in evt:
                            self.cnxqueues[q].put(e + LF)
                    else:
                        self.cnxqueues[q].put(evt + LF)

            readable, writable, exceptional = select.select(self.inputs, self.outputs, self.inputs)
            for s in readable:
                if s is self.server:
                    cnx, caddr = s.accept()
                    logger.debug("New connection from %s", caddr)
                    cnx.setblocking(0)
                    self.inputs.append(cnx)
                    self.outputs.append(cnx)
                    self.cnxqueues.setdefault(cnx)
                    self.cnxqueues[cnx] = Queue.Queue()
                else:
                    try:
                        data = s.recv(1024)
                        caddr = s.getpeername()
                    except Exception as e:	#disconnected
                        logger.error("%s : %s", repr(e.__class__), str(e))
                        data = 'QUIT'
                    if data != "" :
                        logger.debug('Receiving "%s" from %s',data,caddr)
                    if data.upper() == 'QUIT':
                        self.closecnx(s)
            for s in writable:
                if s not in self.outputs: break
                try:
                    next_msg = self.cnxqueues[s].get()
                    s.send(next_msg)                   
                except QEmpty:
                    pass
            for s in exceptional:
                self.closecnx(s)

    def stops(self):
        for s in self.outputs:
            self.closecnx(s)
        self.server.close()
        self.stop = True
        print ('TCPRXServer shutting down...')
